// Copyright 2011 Mark Cavage <mcavage@gmail.com> All rights reserved.

var HttpCodes = require('./http_codes');
var RestCodes = require('./rest_codes');


module.exports = {

  createThrottle: function(options) {
    if (!options) throw new TypeError('options is required');
    if (!options.requests) throw new TypeError('options.requests is required');
    if (!options.seconds) throw new TypeError('options.seconds is required');
    if (!options.ip && !options.username ||
        options.ip && options.username)
      throw new TypeError('options.ip ^ options.username');

    var requests = options.requests;
    var seconds = options.seconds;
    var checked = new Date().getTime();
    var hash = {};

    return function(req, res, next) {
      var attr;
      if (options.ip) {
        attr = req.connection.remoteAddress;
      } else if (options.username) {
        if (options.username === '*' || options.username === true) {
          attr = req.username;
        } else {
          if (options.username !== req.username)
            return next();

          attr = options.username;
        }
      } else {
        return next();
      }

      if (!hash[attr])
        hash[attr] = options.requests;

      var now = new Date().getTime();
      var delta = now - checked;
      checked = now;
      hash[attr] += delta * (requests/seconds);
      if (hash[attr] > requests) {
        hash[attr] = requests;
      } else if (hash[attr] < 1.0) {
        return res.sendError(newError({
          httpCode: HttpCodes.Forbidden,
          restCode: RestCodes.RequestThrottled,
          message: 'You have exceeded ' + requests + 'r/' + seconds + 's'
        }));
      } else {
        hash[attr] -= 1.0;
        return next();
      }
    };
  }

};
